import time

import tornado.gen
import tornado.ioloop
import tornado.web

import pytest

from test import MESSAGE, PAUSE_TIME


class ExpectedError(RuntimeError):

    """
    A dedicated error type for raising from tests to verify a fixture was run.
    """


@pytest.fixture
def mynumber():
    return 42


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write(MESSAGE)


def _pause_coro(period):
    return tornado.gen.sleep(period)


async def pause():
    await _pause_coro(PAUSE_TIME)


def test_plain_function():
    # non-coroutine test function without fixtures
    assert True


def test_plain_function_with_fixture(mynumber):
    # non-coroutine test function that uses a fixture
    assert mynumber == 42


async def nontest_coroutine(io_loop):
    # Non-test coroutine function that shouldn't be run
    assert False


def nontest_function(io_loop):
    # Non-test function that shouldn't be run
    assert False


async def test_pause(io_loop):
    start = time.time()
    await pause()
    elapsed = time.time() - start
    assert elapsed >= PAUSE_TIME


async def test_http_client_fetch(http_client, http_server, http_server_port):
    url = "http://localhost:%s/" % http_server_port[1]
    resp = await http_client.fetch(url)
    assert resp.code == 200
    assert resp.body.decode("utf8") == MESSAGE


async def test_http_server_client_fetch(http_server_client):
    resp = await http_server_client.fetch("/")
    assert resp.code == 200
    assert resp.body.decode("utf8") == MESSAGE


@pytest.mark.xfail(raises=ExpectedError)
def test_expected_noncoroutine_fail():
    raise ExpectedError()


@pytest.mark.xfail(raises=ExpectedError)
async def test_expected_coroutine_fail_no_ioloop():
    """A coroutine test without an io_loop param."""
    raise ExpectedError()


@pytest.mark.xfail(raises=ExpectedError)
async def test_expected_coroutine_fail_io_loop(io_loop):
    """A coroutine test with an io_loop param."""
    raise ExpectedError()


@pytest.mark.xfail(strict=True, raises=tornado.ioloop.TimeoutError)
@pytest.mark.timeout(seconds=0.1)
async def test_timeout(io_loop):
    await _pause_coro(0.15)



import asyncio

import pytest


@pytest.fixture
def io_loop():
    yield asyncio.SelectorEventLoop()


@pytest.mark.xfail(type=TypeError)
async def test_bad_io_loop_fixture(io_loop):
    assert False  # won't be run



import sys
import os

from setuptools import find_packages, setup
from setuptools.command.test import test


here_dir = os.path.abspath(os.path.dirname(__file__))


# require python-3.5+, since we only support the native coroutine 'async def'
# style for tests that were introduced in python 3.5.
if sys.version_info < (3, 5):
    print("pytest-tornasync requires Python 3.5 or newer")
    sys.exit(1)


def read(*filenames):
    buf = []
    for filename in filenames:
        filepath = os.path.join(here_dir, filename)
        try:
            with open(filepath) as f:
                buf.append(f.read())
        except FileNotFoundError:
            pass
    return '\n\n'.join(buf)


class PyTest(test):

    def finalize_options(self):
        test.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        status = pytest.main(self.test_args)
        sys.exit(status)


_reqs = ['pytest>=3.0', 'tornado>=5.0']


setup(
    name='pytest-tornasync',
    version='0.6.0.post2',
    license='http://www.opensource.org/licenses/mit-license.php',
    url='https://github.com/eukaryote/pytest-tornasync',
    description='py.test plugin for testing Python 3.5+ Tornado code',
    long_description=read('README.rst', 'CHANGES.rst'),
    keywords='testing py.test tornado',
    author='Calvin Smith',
    author_email='sapientdust+pytest-tornasync@gmail.com',
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    platforms='any',
    cmdclass={'test': PyTest},
    install_requires=_reqs,
    tests_require=_reqs,
    test_suite='test',
    data_files=[("", ["LICENSE"])],
    entry_points={
        'pytest11': ['tornado = pytest_tornasync.plugin'],
    },
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Framework :: Pytest',
        'Topic :: Software Development :: Testing',
    ] + [
       ("Programming Language :: Python :: %s" % x)
       for x in "3 3.5 3.6 3.7".split()
    ]
)
